Package: atena
Type: Package
Title: Analysis of Transposable Elements
Version: 1.19.0
Authors@R: c(
    person("Beatriz", "Calvo-Serra", email="beatriz.calvo@upf.edu",
           role=c("aut")),
    person("Robert", "Castelo", email="robert.castelo@upf.edu",
           role=c("aut", "cre")))
Description: Quantify expression of transposable elements (TEs) from
        RNA-seq data through different methods, including ERVmap,
        TEtranscripts and Telescope. A common interface is provided to
        use each of these methods, which consists of building a
        parameter object, calling the quantification function with this
        object and getting a SummarizedExperiment object as output
        container of the quantified expression profiles. The
        implementation allows one to quantify TEs and gene transcripts
        in an integrated manner.
License: Artistic-2.0
Encoding: UTF-8
Depends: R (>= 4.3.0), SummarizedExperiment
Imports: methods, stats, Matrix, BiocGenerics, MatrixGenerics,
        BiocParallel, S4Vectors, IRanges, Seqinfo, GenomicFeatures,
        GenomicRanges, GenomicAlignments, Rsamtools, GenomeInfoDb,
        SQUAREM, sparseMatrixStats, AnnotationHub, matrixStats, cli
Suggests: covr, BiocStyle, knitr, rmarkdown, RUnit,
        TxDb.Dmelanogaster.UCSC.dm6.ensGene, RColorBrewer
biocViews: Transcription, Transcriptomics, RNASeq, Sequencing,
        Preprocessing, Software, GeneExpression, Coverage,
        DifferentialExpression, FunctionalGenomics
VignetteBuilder: knitr
URL: https://github.com/rcastelo/atena
BugReports: https://github.com/rcastelo/atena/issues
RoxygenNote: 7.3.2
Collate: 'AllGenerics.R' 'AllClasses.R' 'ERVmap.R' 'TEtranscripts.R'
        'Telescope.R' 'annotations.R' 'atena.R' 'atenaMethod.R'
        'overlappingModes.R' 'qtex.R' 'utils.R' 'zzz.R'
Config/pak/sysreqs: make libbz2-dev libicu-dev liblzma-dev libpng-dev
        libxml2-dev libssl-dev xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:56:57 UTC
RemoteUrl: https://github.com/bioc/atena
RemoteRef: HEAD
RemoteSha: 4524201929394e9a44ff4ffe5d3da45d16e553d6
NeedsCompilation: no
Packaged: 2026-07-04 05:38:37 UTC; root
Author: Beatriz Calvo-Serra [aut],
  Robert Castelo [aut, cre]
Maintainer: Robert Castelo <robert.castelo@upf.edu>
Built: R 4.6.1; ; 2026-07-04 06:14:20 UTC; unix
