# Generated by roxygen2: do not edit by hand

export(ERVmapParam)
export(OneCodeToFindThemAll)
export(TEtranscriptsParam)
export(TelescopeParam)
export(annotaTEs)
export(atenaParam)
export(getDNAtransposons)
export(getLINEs)
export(getLTRs)
export(getSINEs)
export(ovIntersectionStrict)
export(ovUnion)
export(rmskatenaparser)
export(rmskbasicparser)
export(rmskidentity)
exportClasses(ERVmapParam)
exportClasses(QuantifyParam)
exportClasses(TEtranscriptsParam)
exportClasses(TelescopeParam)
exportClasses(atenaParam)
exportMethods(features)
exportMethods(path)
exportMethods(qtex)
exportMethods(show)
importClassesFrom(GenomicAlignments,GAlignments)
importClassesFrom(GenomicRanges,GenomicRanges_OR_GenomicRangesList)
importClassesFrom(Matrix,lgCMatrix)
importClassesFrom(Rsamtools,BamFileList)
importFrom(AnnotationHub,AnnotationHub)
importFrom(AnnotationHub,query)
importFrom(BiocGenerics,basename)
importFrom(BiocGenerics,path)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(GenomeInfoDb,"seqlevelsStyle<-")
importFrom(GenomeInfoDb,keepSeqlevels)
importFrom(GenomeInfoDb,seqlevelsStyle)
importFrom(GenomicAlignments,cigar)
importFrom(GenomicAlignments,explodeCigarOpLengths)
importFrom(GenomicAlignments,extractAlignmentRangesOnReference)
importFrom(GenomicAlignments,findOverlaps)
importFrom(GenomicAlignments,first)
importFrom(GenomicAlignments,qwidth)
importFrom(GenomicAlignments,readGAlignmentPairs)
importFrom(GenomicAlignments,readGAlignments)
importFrom(GenomicAlignments,readGAlignmentsList)
importFrom(GenomicAlignments,second)
importFrom(GenomicAlignments,seqnames)
importFrom(GenomicAlignments,start)
importFrom(GenomicFeatures,features)
importFrom(GenomicRanges,"mcols<-")
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,granges)
importFrom(GenomicRanges,mcols)
importFrom(GenomicRanges,pintersect)
importFrom(GenomicRanges,reduce)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,start)
importFrom(GenomicRanges,strand)
importFrom(GenomicRanges,width)
importFrom(IRanges,CharacterList)
importFrom(IRanges,IRanges)
importFrom(IRanges,IRangesList)
importFrom(IRanges,findOverlapPairs)
importFrom(IRanges,mean)
importFrom(IRanges,ranges)
importFrom(IRanges,relist)
importFrom(IRanges,unique)
importFrom(Matrix,Matrix)
importFrom(Matrix,colSums)
importFrom(Matrix,rowSums)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,summary)
importFrom(Matrix,t)
importFrom(Matrix,which)
importFrom(MatrixGenerics,rowRanges)
importFrom(Rsamtools,"asMates<-")
importFrom(Rsamtools,"yieldSize<-")
importFrom(Rsamtools,BamFileList)
importFrom(Rsamtools,ScanBamParam)
importFrom(Rsamtools,asMates)
importFrom(Rsamtools,bamFlagTest)
importFrom(Rsamtools,scanBamFlag)
importFrom(Rsamtools,scanBamHeader)
importFrom(Rsamtools,yieldSize)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Hits)
importFrom(S4Vectors,List)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,countQueryHits)
importFrom(S4Vectors,countSubjectHits)
importFrom(S4Vectors,decode)
importFrom(S4Vectors,first)
importFrom(S4Vectors,from)
importFrom(S4Vectors,isSorted)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,nLnode)
importFrom(S4Vectors,nRnode)
importFrom(S4Vectors,nrow)
importFrom(S4Vectors,pc)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,rownames)
importFrom(S4Vectors,runValue)
importFrom(S4Vectors,second)
importFrom(S4Vectors,split)
importFrom(S4Vectors,subjectHits)
importFrom(S4Vectors,to)
importFrom(SQUAREM,squarem)
importFrom(Seqinfo,"genome<-")
importFrom(Seqinfo,"seqinfo<-")
importFrom(Seqinfo,"seqlevels<-")
importFrom(Seqinfo,genome)
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(matrixStats,logSumExp)
importFrom(methods,as)
importFrom(methods,formalArgs)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(sparseMatrixStats,colSums2)
importFrom(sparseMatrixStats,rowMaxs)
importFrom(sparseMatrixStats,rowSums2)
importFrom(stats,aggregate)
importFrom(stats,median)
importFrom(stats,runif)
importFrom(stats,setNames)
